@@ -22,6 +22,7 @@
 #include <stdio.h>
 #include <string.h>
 
+#include "opkg_conf.h"
 #include "opkg_message.h"
 #include "opkg_archive.h"
 #include "file_util.h"
@@ -121,6 +122,9 @@ static int copy_to_stream(struct archive *a, FILE * stream)
     int eof;
     size_t len = EXTRACT_BUFFER_LEN;
 
+    if (archive_format(a) == ARCHIVE_FORMAT_EMPTY)
+        return 0;
+
     buffer = xmalloc(len);
 
     while (1) {
@@ -412,8 +416,8 @@ static int extract_entry(struct archive *a, struct archive_entry *entry,
         break;
 
     case ARCHIVE_WARN:
-        opkg_msg(NOTICE, "Warning when extracting archive entry: %s\n",
-                 archive_error_string(a));
+        opkg_msg(NOTICE, "Warning when extracting archive entry '%s': %s\n",
+                 archive_entry_pathname(entry), archive_error_string(a));
         break;
 
     case ARCHIVE_RETRY:
@@ -437,7 +441,7 @@ static int extract_entry(struct archive *a, struct archive_entry *entry,
 /* Extract all files in an archive to the filesystem under the path given by
  * dest. Returns 0 on success or <0 on error.
  */
-static int extract_all(struct archive *a, const char *dest, int flags)
+static int extract_all(struct archive *a, const char *dest, int flags, long unsigned int *size)
 {
     struct archive *disk;
     struct archive_entry *entry;
@@ -470,6 +474,8 @@ static int extract_all(struct archive *a, const char *dest, int flags)
         r = extract_entry(a, entry, disk);
         if (r < 0)
             goto err_cleanup;
+	else if (size)
+            *size += archive_entry_size(entry);
     }
 
     r = ARCHIVE_OK;
@@ -497,14 +503,30 @@ static struct archive *open_outer(const char *filename)
                  archive_error_string(outer));
         goto err_cleanup;
     }
-
-    r = archive_read_open_filename(outer, filename, EXTRACT_BUFFER_LEN);
+    r = archive_read_support_filter_gzip(outer);
+    if (r == ARCHIVE_WARN) {
+        /* libarchive returns ARCHIVE_WARN if the filter is provided by
+         * an external program.
+         */
+        opkg_msg(INFO, "Gzip support provided by external program.\n");
+    } else if (r != ARCHIVE_OK) {
+        opkg_msg(ERROR, "Gzip format not supported.\n");
+        goto err_cleanup;
+    }
+    r = archive_read_support_format_tar(outer);
     if (r != ARCHIVE_OK) {
-        opkg_msg(ERROR, "Failed to open package '%s': %s\n", filename,
-                 archive_error_string(outer));
+        opkg_msg(ERROR, "Tar format not supported: %s\n",
+             archive_error_string(outer));
         goto err_cleanup;
     }
 
+    r = archive_read_open_filename(outer, filename, EXTRACT_BUFFER_LEN);
+    if (r != ARCHIVE_OK)
+    {
+        opkg_msg(ERROR, "Failed to open package '%s': %s\n", filename,
+                    archive_error_string(outer));
+        goto err_cleanup;
+    }
     return outer;
 
  err_cleanup:
@@ -541,6 +563,32 @@ static struct archive *open_inner(struct archive *outer)
         goto err_cleanup;
     }
 
+#if HAVE_XZ
+    r = archive_read_support_filter_xz(inner);
+    if (r == ARCHIVE_WARN) {
+        /* libarchive returns ARCHIVE_WARN if the filter is provided by
+         * an external program.
+         */
+        opkg_msg(INFO, "Xz support provided by external program.\n");
+    } else if (r != ARCHIVE_OK) {
+        opkg_msg(ERROR, "Xz format not supported.\n");
+        goto err_cleanup;
+    }
+#endif
+
+#if HAVE_BZIP2
+    r = archive_read_support_filter_bzip2(inner);
+    if (r == ARCHIVE_WARN) {
+        /* libarchive returns ARCHIVE_WARN if the filter is provided by
+         * an external program.
+         */
+        opkg_msg(INFO, "Bzip2 support provided by external program.\n");
+    } else if (r != ARCHIVE_OK) {
+        opkg_msg(ERROR, "Bzip2 format not supported.\n");
+        goto err_cleanup;
+    }
+#endif
+
     r = archive_read_support_format_tar(inner);
     if (r != ARCHIVE_OK) {
         opkg_msg(ERROR, "Tar format not supported: %s\n",
@@ -654,6 +702,13 @@ static struct archive *open_compressed_file(const char *filename)
         goto err_cleanup;
     }
 
+    r = archive_read_support_format_empty(ar);
+    if (r != ARCHIVE_OK) {
+        opkg_msg(ERROR, "Empty format not supported: %s\n",
+                 archive_error_string(ar));
+        goto err_cleanup;
+    }
+
     /* Open input file and prepare for reading. */
     r = archive_read_open_filename(ar, filename, EXTRACT_BUFFER_LEN);
     if (r != ARCHIVE_OK) {
@@ -680,6 +735,16 @@ struct opkg_ar *ar_open_pkg_control_archive(const char *filename)
     ar = (struct opkg_ar *)xmalloc(sizeof(struct opkg_ar));
 
     ar->ar = extract_outer(filename, "control.tar.gz");
+#if HAVE_XZ
+    if (!ar->ar) {
+        ar->ar = extract_outer(filename, "control.tar.xz");
+    }
+#endif
+#if HAVE_BZIP2
+    if (!ar->ar) {
+        ar->ar = extract_outer(filename, "control.tar.bz2");
+    }
+#endif
     if (!ar->ar) {
         free(ar);
         return NULL;
@@ -702,6 +767,16 @@ struct opkg_ar *ar_open_pkg_data_archive(const char *filename)
     ar = (struct opkg_ar *)xmalloc(sizeof(struct opkg_ar));
 
     ar->ar = extract_outer(filename, "data.tar.gz");
+#if HAVE_XZ
+    if (!ar->ar) {
+        ar->ar = extract_outer(filename, "data.tar.xz");
+    }
+#endif
+#if HAVE_BZIP2
+    if (!ar->ar) {
+        ar->ar = extract_outer(filename, "data.tar.bz2");
+    }
+#endif
     if (!ar->ar) {
         free(ar);
         return NULL;
@@ -714,7 +789,10 @@ struct opkg_ar *ar_open_pkg_data_archive(const char *filename)
      * ARCHIVE_EXTRACT_XATTR).
      */
     ar->extract_flags = ARCHIVE_EXTRACT_OWNER | ARCHIVE_EXTRACT_PERM |
-        ARCHIVE_EXTRACT_TIME | ARCHIVE_EXTRACT_UNLINK;
+        ARCHIVE_EXTRACT_TIME | ARCHIVE_EXTRACT_UNLINK | ARCHIVE_EXTRACT_NO_OVERWRITE;
+
+    if (opkg_config->ignore_uid)
+        ar->extract_flags &= ~ARCHIVE_EXTRACT_OWNER;
 
     return ar;
 }
@@ -723,6 +801,7 @@ struct opkg_ar *ar_open_compressed_file(const char *filename)
 {
     struct opkg_ar *ar;
     struct archive_entry *entry;
+    int eof;
 
     ar = (struct opkg_ar *)xmalloc(sizeof(struct opkg_ar));
 
@@ -737,8 +816,8 @@ struct opkg_ar *ar_open_compressed_file(const char *filename)
      * header. We skip over this header here so that the caller doesn't need
      * to know about it.
      */
-    entry = read_header(ar->ar, NULL);
-    if (!entry)
+    entry = read_header(ar->ar, &eof);
+    if (!entry && !eof)
         goto err_cleanup;
 
     return ar;
@@ -767,9 +846,9 @@ int ar_extract_paths_to_stream(struct opkg_ar *ar, FILE * stream)
     return extract_paths_to_stream(ar->ar, stream);
 }
 
-int ar_extract_all(struct opkg_ar *ar, const char *prefix)
+int ar_extract_all(struct opkg_ar *ar, const char *prefix, long unsigned int *size)
 {
-    return extract_all(ar->ar, prefix, ar->extract_flags);
+    return extract_all(ar->ar, prefix, ar->extract_flags, size);
 }
 
 void ar_close(struct opkg_ar *ar)
