@@ -54,10 +54,16 @@
 static void print_pkg(pkg_t * pkg)
 {
     char *version = pkg_version_str_alloc(pkg);
+    printf("%s - %s", pkg->name, version);
+    if (opkg_config->size) {
+	if(pkg->state_status == SS_INSTALLED || pkg->state_status == SS_UNPACKED)
+	    printf(" - %lu", pkg->installed_size);
+	else
+	    printf(" - %lu", pkg->size);
+	}
     if (pkg->description)
-        printf("%s - %s - %s\n", pkg->name, version, pkg->description);
-    else
-        printf("%s - %s\n", pkg->name, version);
+        printf(" - %s", pkg->description);
+    printf("\n");
     free(version);
 }
 
@@ -151,9 +157,6 @@ static int opkg_update_cmd(int argc, char **argv)
             iter = void_list_next(&opkg_config->pkg_src_list, iter)) {
         src = (pkg_src_t *) iter->data;
 
-        if (src->extra_data && !strcmp(src->extra_data, "__dummy__ "))
-            continue;
-
         err = pkg_src_update(src);
         if (err)
             failures++;
@@ -466,6 +469,11 @@ static int opkg_download_cmd(int argc, char **argv)
             continue;
         }
 
+        if (opkg_config->noaction) {
+            opkg_msg(NOTICE, "Not downloading %s (--noaction specified).\n", pkg->name);
+            continue;
+        }
+
         r = opkg_download_pkg_to_dir(pkg, ".");
         if (r != 0) {
             err = -1;
@@ -479,7 +487,7 @@ static int opkg_download_cmd(int argc, char **argv)
     return err;
 }
 
-static int opkg_list_cmd(int argc, char **argv)
+static int opkg_list_find_cmd(int argc, char **argv, int use_desc)
 {
     unsigned int i;
     pkg_vec_t *available;
@@ -495,7 +503,8 @@ static int opkg_list_cmd(int argc, char **argv)
     for (i = 0; i < available->len; i++) {
         pkg = available->pkgs[i];
         /* if we have package name or pattern and pkg does not match, then skip it */
-        if (pkg_name && fnmatch(pkg_name, pkg->name, 0))
+        if (pkg_name && fnmatch(pkg_name, pkg->name, 0) &&
+           (!use_desc || !pkg->description || fnmatch(pkg_name, pkg->description, 0)))
             continue;
         print_pkg(pkg);
     }
@@ -504,6 +513,16 @@ static int opkg_list_cmd(int argc, char **argv)
     return 0;
 }
 
+static int opkg_list_cmd(int argc, char **argv)
+{
+    return opkg_list_find_cmd(argc, argv, 0);
+}
+
+static int opkg_find_cmd(int argc, char **argv)
+{
+    return opkg_list_find_cmd(argc, argv, 1);
+}
+
 static int opkg_list_installed_cmd(int argc, char **argv)
 {
     unsigned int i;
@@ -515,7 +534,7 @@ static int opkg_list_installed_cmd(int argc, char **argv)
         pkg_name = argv[0];
     }
     available = pkg_vec_alloc();
-    pkg_hash_fetch_all_installed(available);
+    pkg_hash_fetch_all_installed(available, 0);
     pkg_vec_sort(available, pkg_compare_names);
     for (i = 0; i < available->len; i++) {
         pkg = available->pkgs[i];
@@ -543,7 +562,7 @@ static int opkg_list_changed_conffiles_cmd(int argc, char **argv)
         pkg_name = argv[0];
     }
     available = pkg_vec_alloc();
-    pkg_hash_fetch_all_installed(available);
+    pkg_hash_fetch_all_installed(available, 0);
     pkg_vec_sort(available, pkg_compare_names);
     for (i = 0; i < available->len; i++) {
         pkg = available->pkgs[i];
@@ -572,13 +591,14 @@ static int opkg_list_upgradable_cmd(int argc, char **argv)
     opkg_msg(ERROR,"list-upgradable command not available with external solver enabled\n");
     return -1;
 #else
-    struct active_list *head = prepare_upgrade_list();
-    struct active_list *node = NULL;
     pkg_t *_old_pkg, *_new_pkg;
-    char *old_v, *new_v;
-    for (node = active_list_next(head, head); node;
-            node = active_list_next(head, node)) {
-        _old_pkg = list_entry(node, pkg_t, list);
+    LIST_HEAD(head);
+
+    prepare_upgrade_list(&head);
+
+    list_for_each_entry(_old_pkg, &head, list) {
+        char *old_v, *new_v;
+
         _new_pkg = pkg_hash_fetch_best_installation_candidate_by_name(_old_pkg->name);
         if (_new_pkg == NULL)
             continue;
@@ -588,7 +608,6 @@ static int opkg_list_upgradable_cmd(int argc, char **argv)
         free(old_v);
         free(new_v);
     }
-    active_list_head_delete(head);
     return 0;
 #endif
 }
@@ -607,7 +626,7 @@ static int opkg_info_status_cmd(int argc, char **argv, int installed_only)
 
     available = pkg_vec_alloc();
     if (installed_only)
-        pkg_hash_fetch_all_installed(available);
+        pkg_hash_fetch_all_installed(available, 1);
     else
         pkg_hash_fetch_available(available);
 
@@ -692,12 +711,48 @@ static int opkg_remove_cmd(int argc, char **argv)
     return err;
 }
 
+/** \brief opkg_is_state_flag: tells whether the flag is state flag
+ *
+ * \param flag
+ * \return 1 is success, 0 otherwise
+ */
+static int opkg_is_state_flag(const char* flag)
+{
+    return (
+        (strcmp(flag, "hold") == 0) ||
+        (strcmp(flag, "noprune") == 0) ||
+        (strcmp(flag, "ok") == 0) ||
+        (strcmp(flag, "user") == 0)
+    );
+}
+
+/** \brief opkg_is_state_status_flag: tells whether the flag is status flag
+ *
+ * \param flag
+ * \return 1 is success, 0 otherwise
+ */
+static int opkg_is_state_status_flag(const char* flag)
+{
+    return (
+        (strcmp(flag, "installed") == 0) ||
+        (strcmp(flag, "unpacked") == 0)
+    );
+}
+
 static int opkg_flag_cmd(int argc, char **argv)
 {
     int i;
     pkg_t *pkg;
     const char *flags = argv[0];
 
+    int is_state_flag = opkg_is_state_flag(flags);
+    int is_state_status_flag = opkg_is_state_status_flag(flags);
+
+    if ((!is_state_flag) && (!is_state_status_flag)) {
+        opkg_msg(ERROR, "Invalid flag: %s.\n", flags);
+        return -1;
+    }
+
     signal(SIGINT, sigint_handler);
 
     for (i = 1; i < argc; i++) {
@@ -712,8 +767,7 @@ static int opkg_flag_cmd(int argc, char **argv)
             opkg_msg(ERROR, "Package %s is not installed.\n", argv[i]);
             continue;
         }
-        int is_state_flag = ((strcmp(flags, "hold") == 0) || (strcmp(flags, "noprune") == 0)
-                || (strcmp(flags, "user") == 0) || (strcmp(flags, "ok") == 0));
+
         if (is_state_flag) {
             pkg->state_flag = pkg_state_flag_from_str(flags);
         }
@@ -722,9 +776,7 @@ static int opkg_flag_cmd(int argc, char **argv)
          * Useful if a package is installed in an offline_root, and
          * should be configured by opkg configure at a later date.
          */
-        int is_state_status = ((strcmp(flags, "installed") == 0)
-                || (strcmp(flags, "unpacked") == 0));
-        if (is_state_status) {
+        if (is_state_status_flag) {
             pkg->state_status = pkg_state_status_from_str(flags);
         }
 
@@ -785,7 +837,7 @@ static int opkg_depends_cmd(int argc, char **argv)
     if (opkg_config->query_all)
         pkg_hash_fetch_available(available_pkgs);
     else
-        pkg_hash_fetch_all_installed(available_pkgs);
+        pkg_hash_fetch_all_installed(available_pkgs, 0);
 
     for (i = 0; i < argc; i++) {
         for (j = 0; j < available_pkgs->len; j++) {
@@ -867,7 +919,7 @@ static int opkg_what_depends_conflicts_replaces_cmd(enum depend_type what_field_
     if (opkg_config->query_all)
         pkg_hash_fetch_available(available_pkgs);
     else
-        pkg_hash_fetch_all_installed(available_pkgs);
+        pkg_hash_fetch_all_installed(available_pkgs, 0);
 
     /* mark the root set */
     pkg_vec_clear_marks(available_pkgs);
@@ -978,7 +1030,7 @@ static int opkg_whatprovides_cmd(int argc, char **argv)
         if (opkg_config->query_all)
             pkg_hash_fetch_available(available_pkgs);
         else
-            pkg_hash_fetch_all_installed(available_pkgs);
+            pkg_hash_fetch_all_installed(available_pkgs, 0);
         for (i = 0; i < argc; i++) {
             const char *target = argv[i];
             unsigned int j;
@@ -1024,7 +1076,7 @@ static int opkg_search_cmd(int argc, char **argv)
     }
 
     installed = pkg_vec_alloc();
-    pkg_hash_fetch_all_installed(installed);
+    pkg_hash_fetch_all_installed(installed, 0);
     pkg_vec_sort(installed, pkg_compare_names);
 
     for (i = 0; i < installed->len; i++) {
@@ -1107,6 +1159,8 @@ static opkg_cmd_t cmds[] = {
     {"files", 1, (opkg_cmd_fun_t) opkg_files_cmd, PFM_DESCRIPTION | PFM_SOURCE},
     {"search", 1, (opkg_cmd_fun_t) opkg_search_cmd,
         PFM_DESCRIPTION | PFM_SOURCE},
+    {"find", 1, (opkg_cmd_fun_t) opkg_find_cmd,
+        PFM_DESCRIPTION | PFM_SOURCE},
     {"download", 1, (opkg_cmd_fun_t) opkg_download_cmd,
         PFM_DESCRIPTION | PFM_SOURCE},
     {"compare_versions", 1, (opkg_cmd_fun_t) opkg_compare_versions_cmd, 0},
