@@ -47,6 +47,7 @@ opkg_conf_t *opkg_config = &_conf;
  */
 static opkg_option_t options[] = {
     {"cache_dir", OPKG_OPT_TYPE_STRING, &_conf.cache_dir},
+    {"intercepts_dir", OPKG_OPT_TYPE_STRING, &_conf.intercepts_dir},
     {"lists_dir", OPKG_OPT_TYPE_STRING, &_conf.lists_dir},
     {"lock_file", OPKG_OPT_TYPE_STRING, &_conf.lock_file},
     {"info_dir", OPKG_OPT_TYPE_STRING, &_conf.info_dir},
@@ -61,6 +62,7 @@ static opkg_option_t options[] = {
     {"force_reinstall", OPKG_OPT_TYPE_BOOL, &_conf.force_reinstall},
     {"force_space", OPKG_OPT_TYPE_BOOL, &_conf.force_space},
     {"force_postinstall", OPKG_OPT_TYPE_BOOL, &_conf.force_postinstall},
+    {"force_checksum", OPKG_OPT_TYPE_BOOL, &_conf.force_checksum},
     {"autoremove", OPKG_OPT_TYPE_BOOL, &_conf.autoremove},
     {"check_signature", OPKG_OPT_TYPE_BOOL, &_conf.check_signature},
     {"check_pkg_signature", OPKG_OPT_TYPE_BOOL, &_conf.check_pkg_signature},
@@ -72,6 +74,7 @@ static opkg_option_t options[] = {
     {"test", OPKG_OPT_TYPE_BOOL, &_conf.noaction},
     {"noaction", OPKG_OPT_TYPE_BOOL, &_conf.noaction},
     {"download_only", OPKG_OPT_TYPE_BOOL, &_conf.download_only},
+    {"download_first", OPKG_OPT_TYPE_BOOL, &_conf.download_first}, /* Not available on internal solver */
     {"nodeps", OPKG_OPT_TYPE_BOOL, &_conf.nodeps},
     {"no_install_recommends", OPKG_OPT_TYPE_BOOL, &_conf.no_install_recommends},
     {"offline_root", OPKG_OPT_TYPE_STRING, &_conf.offline_root},
@@ -79,12 +82,17 @@ static opkg_option_t options[] = {
     {"proxy_passwd", OPKG_OPT_TYPE_STRING, &_conf.proxy_passwd},
     {"proxy_user", OPKG_OPT_TYPE_STRING, &_conf.proxy_user},
     {"query-all", OPKG_OPT_TYPE_BOOL, &_conf.query_all},
+    {"size", OPKG_OPT_TYPE_BOOL, &_conf.size},
     {"tmp_dir", OPKG_OPT_TYPE_STRING, &_conf.tmp_dir},
     {"volatile_cache", OPKG_OPT_TYPE_BOOL, &_conf.volatile_cache},
     {"verbosity", OPKG_OPT_TYPE_INT, &_conf.verbosity},
     {"overwrite_no_owner", OPKG_OPT_TYPE_BOOL, &_conf.overwrite_no_owner},
     {"combine", OPKG_OPT_TYPE_BOOL, &_conf.combine},
     {"cache_local_files", OPKG_OPT_TYPE_BOOL, &_conf.cache_local_files},
+#if defined(HAVE_GPGME)
+    {"gpg_dir", OPKG_OPT_TYPE_STRING, &_conf.gpg_dir},
+    {"gpg_trust_level", OPKG_OPT_TYPE_STRING, &_conf.gpg_trust_level},
+#endif
 #if defined(HAVE_OPENSSL)
     {"signature_ca_file", OPKG_OPT_TYPE_STRING, &_conf.signature_ca_file},
     {"signature_ca_path", OPKG_OPT_TYPE_STRING, &_conf.signature_ca_path},
@@ -165,6 +173,33 @@ static opkg_option_t *opkg_conf_find_option(const char *name)
     return NULL;
 }
 
+static void parse_pkg_src_options_str(pkg_src_options_t *src_options,
+                                      char *options_str)
+{
+    char *token, *value, *src_option;
+
+    /* default value */
+    src_options->disable_sig_check = 0;
+
+    token = strtok(options_str, " ");
+    while (token) {
+        value = strrchr(token, '=');
+        if (value) {
+            /* Remove '=' character */
+            value++;
+            src_option = xstrndup(token, strlen(token) - (strlen(value) + 1));
+            if (strcasecmp(src_option, "trusted") == 0) {
+                if (strcasecmp(value, "yes") == 0)
+                    src_options->disable_sig_check = 1;
+                else
+                    src_options->disable_sig_check = 0;
+            }
+            free(src_option);
+        }
+        token = strtok(NULL, " ");
+    }
+}
+
 int opkg_conf_get_option(char *name, void *value)
 {
     opkg_option_t *o;
@@ -267,7 +302,7 @@ static int opkg_conf_parse_file(const char *filename,
     int r;
     FILE *file;
     regex_t valid_line_re, comment_re;
-#define regmatch_size 14
+#define regmatch_size 16
     regmatch_t regmatch[regmatch_size];
 
     file = fopen(filename, "r");
@@ -288,6 +323,7 @@ static int opkg_conf_parse_file(const char *filename,
                  "^[[:space:]]*(\"([^\"]*)\"|([^[:space:]]*))"
                  "[[:space:]]*(\"([^\"]*)\"|([^[:space:]]*))"
                  "[[:space:]]*(\"([^\"]*)\"|([^[:space:]]*))"
+                 "([[:space:]]+([[]([^]])+[]]))?"
                  "([[:space:]]+([^[:space:]]+))?([[:space:]]+(.*))?[[:space:]]*$",
                  REG_EXTENDED);
     if (err)
@@ -296,6 +332,7 @@ static int opkg_conf_parse_file(const char *filename,
     while (1) {
         char *line;
         char *type, *name, *value, *extra;
+        pkg_src_options_t *src_options = NULL;
 
         line_num++;
 
@@ -341,15 +378,22 @@ static int opkg_conf_parse_file(const char *filename,
 
         extra = NULL;
         if (regmatch[11].rm_so > 0) {
-            if (regmatch[13].rm_so > 0 && regmatch[13].rm_so != regmatch[13].rm_eo)
-                extra = xstrndup(line + regmatch[11].rm_so,
-                             regmatch[13].rm_eo - regmatch[11].rm_so);
+            char *options_str = xstrndup(1 + line + regmatch[11].rm_so,
+                                    (regmatch[11].rm_eo - regmatch[11].rm_so) - 2);
+            src_options = xmalloc(sizeof(pkg_src_options_t));
+            parse_pkg_src_options_str(src_options, options_str);
+            free(options_str);
+        }
+        if (regmatch[13].rm_so > 0) {
+            if (regmatch[15].rm_so > 0 && regmatch[15].rm_so != regmatch[15].rm_eo)
+                extra = xstrndup(line + regmatch[13].rm_so,
+                             regmatch[15].rm_eo - regmatch[13].rm_so);
             else
-                extra = xstrndup(line + regmatch[11].rm_so,
-                             regmatch[11].rm_eo - regmatch[11].rm_so);
+                extra = xstrndup(line + regmatch[13].rm_so,
+                             regmatch[13].rm_eo - regmatch[13].rm_so);
         }
 
-        if (regmatch[13].rm_so != regmatch[13].rm_eo && strncmp(type, "dist", 4) != 0) {
+        if (regmatch[15].rm_so != regmatch[15].rm_eo && strncmp(type, "dist", 4) != 0) {
             opkg_msg(ERROR,
                      "%s:%d: Ignoring config line with trailing garbage: `%s'\n",
                      filename, line_num, line);
@@ -364,7 +408,7 @@ static int opkg_conf_parse_file(const char *filename,
                 opkg_conf_set_option(name, value, 0);
             } else if (strcmp(type, "dist") == 0) {
                 if (!nv_pair_list_find((nv_pair_list_t *) dist_src_list, name)) {
-                    pkg_src_list_append(dist_src_list, name, value, extra, 0);
+                    pkg_src_list_append(dist_src_list, name, value, src_options, extra, 0);
                 } else {
                     opkg_msg(ERROR,
                              "Duplicate dist declaration (%s %s). "
@@ -372,7 +416,7 @@ static int opkg_conf_parse_file(const char *filename,
                 }
             } else if (strcmp(type, "dist/gz") == 0) {
                 if (!nv_pair_list_find((nv_pair_list_t *) dist_src_list, name)) {
-                    pkg_src_list_append(dist_src_list, name, value, extra, 1);
+                    pkg_src_list_append(dist_src_list, name, value, src_options, extra, 1);
                 } else {
                     opkg_msg(ERROR,
                              "Duplicate dist declaration (%s %s). "
@@ -380,7 +424,7 @@ static int opkg_conf_parse_file(const char *filename,
                 }
             } else if (strcmp(type, "src") == 0) {
                 if (!nv_pair_list_find((nv_pair_list_t *) pkg_src_list, name)) {
-                    pkg_src_list_append(pkg_src_list, name, value, extra, 0);
+                    pkg_src_list_append(pkg_src_list, name, value, src_options, NULL, 0);
                 } else {
                     opkg_msg(ERROR,
                              "Duplicate src declaration (%s %s). "
@@ -388,7 +432,7 @@ static int opkg_conf_parse_file(const char *filename,
                 }
             } else if (strcmp(type, "src/gz") == 0) {
                 if (!nv_pair_list_find((nv_pair_list_t *) pkg_src_list, name)) {
-                    pkg_src_list_append(pkg_src_list, name, value, extra, 1);
+                    pkg_src_list_append(pkg_src_list, name, value, src_options, NULL, 1);
                 } else {
                     opkg_msg(ERROR,
                              "Duplicate src declaration (%s %s). "
@@ -416,6 +460,7 @@ static int opkg_conf_parse_file(const char *filename,
         free(type);
         free(name);
         free(value);
+        free(src_options);
         free(extra);
 
  NEXT_LINE:
@@ -613,7 +658,6 @@ int opkg_conf_load(void)
 #if defined(HAVE_PATHFINDER)
     opkg_config->check_x509_path = 1;
 #endif
-
     if (!opkg_config->offline_root)
         opkg_config->offline_root = xstrdup(getenv("OFFLINE_ROOT"));
 
@@ -629,13 +673,14 @@ int opkg_conf_load(void)
         if (r != 0)
             goto err1;
     } else {
-        if (opkg_config->offline_root)
-            sprintf_alloc(&etc_opkg_conf_pattern, "%s/etc/opkg/*.conf",
-                          opkg_config->offline_root);
-        else {
-            const char *conf_file_dir = getenv("OPKG_CONF_DIR");
-            if (conf_file_dir == NULL)
-                conf_file_dir = OPKG_CONF_DEFAULT_CONF_FILE_DIR;
+        const char *conf_file_dir = getenv("OPKG_CONF_DIR");
+        if (conf_file_dir == NULL)
+            conf_file_dir=OPKG_CONF_DEFAULT_CONF_FILE_DIR;
+        if (opkg_config->offline_root) {
+            sprintf_alloc(&etc_opkg_conf_pattern, "%s/%s/*.conf",
+                        opkg_config->offline_root,
+                        conf_file_dir);
+        } else {
             sprintf_alloc(&etc_opkg_conf_pattern, "%s/*.conf", conf_file_dir);
         }
 
@@ -666,6 +711,12 @@ int opkg_conf_load(void)
         globfree(&globbuf);
     }
 
+    /* Option not available on the internal solver since it currently
+     * can't merge transactions, resulting in one solve per operation */
+#if defined(HAVE_SOLVER_INTERNAL)
+    opkg_config->download_first = 0;
+#endif
+
     if (opkg_config->lock_file == NULL)
         opkg_config->lock_file = xstrdup(OPKG_CONF_DEFAULT_LOCK_FILE);
 
@@ -703,6 +754,9 @@ int opkg_conf_load(void)
     hash_table_init("obs-file-hash", &opkg_config->obs_file_hash,
                     OPKG_CONF_DEFAULT_HASH_LEN / 16);
 
+    if (opkg_config->intercepts_dir == NULL)
+        opkg_config->intercepts_dir = xstrdup(DATADIR "/opkg/intercept");
+
     if (opkg_config->lists_dir == NULL)
         opkg_config->lists_dir = xstrdup(OPKG_CONF_DEFAULT_LISTS_DIR);
 
@@ -710,6 +764,11 @@ int opkg_conf_load(void)
         opkg_config->cache_dir = xstrdup(OPKG_CONF_DEFAULT_CACHE_DIR);
 
     if (opkg_config->offline_root) {
+        sprintf_alloc(&tmp, "%s/%s", opkg_config->offline_root,
+                      opkg_config->intercepts_dir);
+        free(opkg_config->intercepts_dir);
+        opkg_config->intercepts_dir = tmp;
+
         sprintf_alloc(&tmp, "%s/%s", opkg_config->offline_root,
                       opkg_config->lists_dir);
         free(opkg_config->lists_dir);
@@ -732,6 +791,35 @@ int opkg_conf_load(void)
     if (opkg_config->signature_type == NULL)
         opkg_config->signature_type = xstrdup(OPKG_CONF_DEFAULT_SIGNATURE_TYPE);
 
+#if defined(HAVE_GPGME)
+    if (opkg_config->gpg_dir == NULL){
+        opkg_config->gpg_dir = xstrdup(OPKG_CONF_GPG_DEFAULT_DIR);
+    }
+
+    if (opkg_config->offline_root) {
+        sprintf_alloc(&tmp, "%s/%s", opkg_config->offline_root,
+                      opkg_config->gpg_dir);
+        free(opkg_config->gpg_dir);
+        opkg_config->gpg_dir = tmp;
+    }
+
+    if (opkg_config->gpg_trust_level == NULL){
+        opkg_config->gpg_trust_level = xstrdup(OPKG_CONF_DEFAULT_GPG_TRUST_LEVEL);
+    }else{
+        /* Verify that the gpg_trust_level is set approriately */
+        if(strncmp(opkg_config->gpg_trust_level,
+                    OPKG_CONF_GPG_TRUST_ONLY,
+                    sizeof(OPKG_CONF_GPG_TRUST_ONLY)) != 0 &&
+           strncmp(opkg_config->gpg_trust_level,
+                    OPKG_CONF_GPG_TRUST_ANY,
+                    sizeof(OPKG_CONF_GPG_TRUST_ANY)) !=0)
+        {
+            opkg_perror(ERROR, "Unrecognized gpg_trust_level %s\n", opkg_config->gpg_trust_level);
+            goto err3;
+        }
+    }
+#endif
+
     /* if no architectures were defined, then default all, noarch, and host architecture */
     if (nv_pair_list_empty(&opkg_config->arch_list)) {
         nv_pair_list_append(&opkg_config->arch_list, "all", "1");
