@@ -28,11 +28,6 @@ extern "C" {
 
 int opkg_install_pkg(pkg_t * pkg, int from_upgrading);
 
-#ifndef HAVE_SOLVER
-int opkg_install_by_name(const char *pkg_name);
-int opkg_install_multiple_by_name(str_list_t * pkg_names);
-#endif
-
 #ifdef __cplusplus
 }
 #endif
