@@ -1,3 +1,4 @@
+/* vi: set expandtab sw=4 sts=4: */
 /* opkg_message.h - the opkg package management system
 
    Copyright (C) 2009 Ubiq Technologies <graham.gower@gmail.com>
@@ -20,27 +21,35 @@
 #include <string.h>
 #include <errno.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 typedef enum {
-	ERROR,	/* error conditions */
-	NOTICE,	/* normal but significant condition */
-	INFO,	/* informational message */
-	DEBUG,	/* debug level message */
-	DEBUG2,	/* more debug level message */
+    ERROR,                  /* error conditions */
+    NOTICE,                 /* normal but significant condition */
+    INFO,                   /* informational message */
+    DEBUG,                  /* debug level message */
+    DEBUG2,                 /* more debug level message */
 } message_level_t;
 
 void free_error_list(void);
 void print_error_list(void);
-void opkg_message(message_level_t level, const char *fmt, ...);
+void opkg_message(message_level_t level, const char *fmt, ...)
+    __attribute__ ((format(printf, 2, 3)));
 
 #define opkg_msg(l, fmt, args...) \
-	do { \
-		if (l == NOTICE) \
-			opkg_message(l, fmt, ##args); \
-		else \
-			opkg_message(l, "%s: "fmt, __FUNCTION__, ##args); \
-	} while (0)
+    do { \
+        if (l == NOTICE) \
+            opkg_message(l, fmt, ##args); \
+        else \
+            opkg_message(l, "%s: "fmt, __FUNCTION__, ##args); \
+    } while (0)
 
 #define opkg_perror(l, fmt, args...) \
-	opkg_msg(l, fmt": %s.\n", ##args, strerror(errno))
+    opkg_msg(l, fmt": %s.\n", ##args, strerror(errno))
 
-#endif /* _OPKG_MESSAGE_H_ */
+#ifdef __cplusplus
+}
+#endif
+#endif                          /* _OPKG_MESSAGE_H_ */
