@@ -102,8 +102,6 @@ static void pkg_init(pkg_t * pkg)
     pkg->suggests_count = 0;
     pkg->recommends_count = 0;
 
-    active_list_init(&pkg->list);
-
     pkg->conflicts = NULL;
     pkg->conflicts_count = 0;
 
@@ -190,10 +188,11 @@ void pkg_deinit(pkg_t * pkg)
     pkg->state_flag = SF_OK;
     pkg->state_status = SS_NOT_INSTALLED;
 
-    active_list_clear(&pkg->list);
-
-    free(pkg->replaces);
-    pkg->replaces = NULL;
+    if (pkg->replaces) {
+        for (i = 0; i < pkg->replaces_count; i++)
+            compound_depend_deinit(&pkg->replaces[i]);
+        free(pkg->replaces);
+    }
 
     if (pkg->depends) {
         unsigned int count = pkg->pre_depends_count + pkg->depends_count
@@ -561,21 +560,7 @@ static const char *pkg_state_status_to_str(pkg_state_status_t ss)
     return "<STATE_STATUS_UNKNOWN>";
 }
 
-pkg_state_status_t pkg_state_status_from_str(const char *str)
-{
-    unsigned int i;
-
-    for (i = 0; i < ARRAY_SIZE(pkg_state_status_map); i++) {
-        if (strcmp(str, pkg_state_status_map[i].str) == 0) {
-            return pkg_state_status_map[i].value;
-        }
-    }
-
-    opkg_msg(ERROR, "Internal error: state_status=%s\n", str);
-    return SS_NOT_INSTALLED;
-}
-
-void pkg_formatted_field(FILE * fp, pkg_t * pkg, const char *field)
+static void pkg_formatted_field(FILE * fp, pkg_t * pkg, const char *field)
 {
     unsigned int i, j;
     char *str;
@@ -673,7 +658,8 @@ void pkg_formatted_field(FILE * fp, pkg_t * pkg, const char *field)
         break;
     case 'i':
     case 'I':
-        if (strcasecmp(field, "Installed-Size") == 0) {
+        if (strcasecmp(field, "Installed-Size") == 0
+		   && pkg->installed_size) {
             fprintf(fp, "Installed-Size: %ld\n", pkg->installed_size);
         } else if (strcasecmp(field, "Installed-Time") == 0
                    && pkg->installed_time) {
@@ -717,11 +703,17 @@ void pkg_formatted_field(FILE * fp, pkg_t * pkg, const char *field)
     case 'r':
     case 'R':
         if (strcasecmp(field, "Replaces") == 0) {
+            struct depend *rdep;
             if (pkg->replaces_count) {
                 fprintf(fp, "Replaces:");
                 for (i = 0; i < pkg->replaces_count; i++) {
-                    fprintf(fp, "%s %s", i == 0 ? "" : ",",
-                            pkg->replaces[i]->name);
+                    rdep = pkg->replaces[i].possibilities[0];
+                    fprintf(fp, "%s %s", i == 0 ? "" : ",", rdep->pkg->name);
+                    if (rdep->version) {
+                        fprintf(fp, " (%s%s)",
+                                constraint_to_str(rdep->constraint),
+                                rdep->version);
+                    }
                 }
                 fprintf(fp, "\n");
             }
@@ -813,6 +805,20 @@ void pkg_formatted_field(FILE * fp, pkg_t * pkg, const char *field)
     opkg_msg(ERROR, "Internal error: field=%s\n", field);
 }
 
+pkg_state_status_t pkg_state_status_from_str(const char *str)
+{
+    unsigned int i;
+
+    for (i = 0; i < ARRAY_SIZE(pkg_state_status_map); i++) {
+        if (strcmp(str, pkg_state_status_map[i].str) == 0) {
+            return pkg_state_status_map[i].value;
+        }
+    }
+
+    opkg_msg(ERROR, "Internal error: state_status=%s\n", str);
+    return SS_NOT_INSTALLED;
+}
+
 void pkg_formatted_info(FILE * fp, pkg_t * pkg)
 {
     pkg_formatted_field(fp, pkg, "Package");
@@ -834,6 +840,7 @@ void pkg_formatted_info(FILE * fp, pkg_t * pkg)
     pkg_formatted_field(fp, pkg, "Conffiles");
     pkg_formatted_field(fp, pkg, "Source");
     pkg_formatted_field(fp, pkg, "Description");
+    pkg_formatted_field(fp, pkg, "Installed-Size");
     pkg_formatted_field(fp, pkg, "Installed-Time");
     pkg_formatted_field(fp, pkg, "Tags");
     fputs("\n", fp);
@@ -857,6 +864,7 @@ void pkg_print_status(pkg_t * pkg, FILE * file)
     pkg_formatted_field(file, pkg, "Essential");
     pkg_formatted_field(file, pkg, "Architecture");
     pkg_formatted_field(file, pkg, "Conffiles");
+    pkg_formatted_field(file, pkg, "Installed-Size");
     pkg_formatted_field(file, pkg, "Installed-Time");
     pkg_formatted_field(file, pkg, "Auto-Installed");
     fputs("\n", file);
@@ -1109,7 +1117,8 @@ str_list_t *pkg_get_installed_files(pkg_t * pkg)
                       pkg->name);
         list_file = fopen(list_file_name, "r");
         if (list_file == NULL) {
-            opkg_perror(ERROR, "Failed to open %s", list_file_name);
+            if (pkg->state_status != SS_HALF_INSTALLED)
+                opkg_perror(ERROR, "Failed to open %s", list_file_name);
             free(list_file_name);
             return pkg->installed_files;
         }
@@ -1303,7 +1312,7 @@ void pkg_info_preinstall_check(void)
 
     /* update the file owner data structure */
     opkg_msg(INFO, "Updating file owner list.\n");
-    pkg_hash_fetch_all_installed(installed_pkgs);
+    pkg_hash_fetch_all_installed(installed_pkgs, 0);
     for (i = 0; i < installed_pkgs->len; i++) {
         pkg_t *pkg = installed_pkgs->pkgs[i];
         str_list_t *installed_files = pkg_get_installed_files(pkg);     /* this causes installed_files to be cached */
@@ -1379,7 +1388,7 @@ int pkg_write_changed_filelists(void)
 
     opkg_msg(INFO, "Saving changed filelists.\n");
 
-    pkg_hash_fetch_all_installed(installed_pkgs);
+    pkg_hash_fetch_all_installed(installed_pkgs, 0);
     for (i = 0; i < installed_pkgs->len; i++) {
         pkg_t *pkg = installed_pkgs->pkgs[i];
         if (pkg->state_flag & SF_FILELIST_CHANGED) {
@@ -1426,6 +1435,8 @@ int pkg_verify(pkg_t * pkg)
         err = opkg_verify_signature(pkg->local_filename, local_sig_filename);
         if (err)
             goto fail;
+
+	opkg_msg(DEBUG, "Signature verification passed for %s.\n", pkg->local_filename);
     }
 
     free(local_sig_filename);
@@ -1433,8 +1444,17 @@ int pkg_verify(pkg_t * pkg)
 
  fail:
     free(local_sig_filename);
-    opkg_msg(NOTICE, "Removing corrupt package file %s.\n",
+    if (!opkg_config->force_checksum)
+    {
+	opkg_msg(NOTICE, "Removing corrupt package file %s.\n",
              pkg->local_filename);
-    unlink(pkg->local_filename);
-    return err;
+	unlink(pkg->local_filename);
+	return err;
+    }
+    else
+    {
+	opkg_msg(NOTICE, "Ignored %s checksum mismatch.\n",
+             pkg->local_filename);
+        return 0;
+    }
 }
