@@ -23,7 +23,6 @@
 
 #include "pkg_vec.h"
 #include "str_list.h"
-#include "active_list.h"
 #include "pkg_src.h"
 #include "pkg_dest.h"
 #include "opkg_conf.h"
@@ -137,7 +136,7 @@ struct pkg {
     unsigned int recommends_count;
     char **suggests_str;
     unsigned int suggests_count;
-    struct active_list list;        /* Used for installing|upgrading */
+    struct list_head list;        /* Used for installing|upgrading */
     compound_depend_t *depends;
 
     char **conflicts_str;
@@ -146,7 +145,7 @@ struct pkg {
 
     char **replaces_str;
     unsigned int replaces_count;
-    abstract_pkg_t **replaces;
+    compound_depend_t *replaces;
 
     char **provides_str;
     unsigned int provides_count;
@@ -203,7 +202,6 @@ int pkg_name_version_and_architecture_compare(const void *a, const void *b);
 int abstract_pkg_name_compare(const void *a, const void *b);
 
 void pkg_formatted_info(FILE * fp, pkg_t * pkg);
-void pkg_formatted_field(FILE * fp, pkg_t * pkg, const char *field);
 
 void set_flags_from_control(pkg_t * pkg);
 
