@@ -20,6 +20,7 @@
 
 #include <stdio.h>
 #include <malloc.h>
+#include <fnmatch.h>
 
 #include "hash_table.h"
 #include "release.h"
@@ -152,7 +153,7 @@ static int dist_hash_add_from_file(pkg_src_t * dist)
                 return -1;
             }
             pkg_src_list_append(&opkg_config->pkg_src_list, subname,
-                                dist->value, "__dummy__", 0);
+                                dist->value, NULL, "__dummy__", 0);
         }
 
         free(list_file);
@@ -274,9 +275,29 @@ abstract_pkg_t *abstract_pkg_fetch_by_name(const char *pkg_name)
     return (abstract_pkg_t *) hash_table_get(&opkg_config->pkg_hash, pkg_name);
 }
 
+void abstract_pkgs_fetch_by_glob(const char *pkg_glob, abstract_pkg_vec_t *apkgs)
+{
+    unsigned int i;
+    hash_table_t *hash = &opkg_config->pkg_hash;
+
+    if (!hash)
+        return;
+
+    for (i = 0; i < hash->n_buckets; i++) {
+        hash_entry_t *hash_entry = (hash->entries + i);
+
+        do {
+            if (hash_entry->key) {
+                if (!fnmatch(pkg_glob, hash_entry->key, 0))
+                    abstract_pkg_vec_insert(apkgs, hash_entry->data);
+            }
+        } while ((hash_entry = hash_entry->next));
+     }
+}
+
 pkg_t *pkg_hash_fetch_best_installation_candidate(abstract_pkg_t * apkg,
                                                   int (*constraint_fcn) (pkg_t * pkg, void *cdata),
-                                                  void *cdata, int quiet)
+                                                  void *cdata, int prefer_installed, int quiet)
 {
     unsigned int i, j;
     unsigned int nprovides = 0;
@@ -419,7 +440,7 @@ pkg_t *pkg_hash_fetch_best_installation_candidate(abstract_pkg_t * apkg,
     if (matching_pkgs->len > 1)
         pkg_vec_sort(matching_pkgs, pkg_name_version_and_architecture_compare);
     if (matching_apkgs->len > 1)
-        abstract_pkg_vec_sort(matching_pkgs, abstract_pkg_name_compare);
+        abstract_pkg_vec_sort(matching_apkgs, abstract_pkg_name_compare);
 
     for (i = 0; i < matching_pkgs->len; i++) {
         pkg_t *matching = matching_pkgs->pkgs[i];
@@ -443,6 +464,11 @@ pkg_t *pkg_hash_fetch_best_installation_candidate(abstract_pkg_t * apkg,
                 } else
                     opkg_msg(DEBUG, "%s %s wins by priority.\n",
                              good_pkg_by_name->name, good_pkg_by_name->version);
+            } else if (good_pkg_by_name && prefer_installed) {
+                int is_installed = good_pkg_by_name->state_status == SS_INSTALLED
+                                   || good_pkg_by_name->state_status == SS_UNPACKED;
+                if (!is_installed)
+                    good_pkg_by_name = matching;
             } else
                 good_pkg_by_name = matching;
         }
@@ -574,7 +600,7 @@ pkg_t *pkg_hash_fetch_best_installation_candidate_by_name(const char *name)
     if (!apkg)
         return NULL;
 
-    return pkg_hash_fetch_best_installation_candidate(apkg, NULL, NULL, 0);
+    return pkg_hash_fetch_best_installation_candidate(apkg, NULL, NULL, 0, 0);
 }
 
 pkg_t *pkg_hash_fetch_by_name_version(const char *pkg_name, const char *version)
@@ -619,7 +645,8 @@ pkg_t *pkg_hash_fetch_installed_by_name_dest(const char *pkg_name,
 
     for (i = 0; i < vec->len; i++) {
         int is_installed = (vec->pkgs[i]->state_status == SS_INSTALLED
-                    || vec->pkgs[i]->state_status == SS_UNPACKED)
+                    || vec->pkgs[i]->state_status == SS_UNPACKED
+                    || vec->pkgs[i]->state_status == SS_HALF_INSTALLED)
                 && vec->pkgs[i]->dest == dest;
         if (is_installed)
             return vec->pkgs[i];
@@ -640,7 +667,8 @@ pkg_t *pkg_hash_fetch_installed_by_name(const char *pkg_name)
 
     for (i = 0; i < vec->len; i++) {
         int is_installed = vec->pkgs[i]->state_status == SS_INSTALLED
-            || vec->pkgs[i]->state_status == SS_UNPACKED;
+            || vec->pkgs[i]->state_status == SS_UNPACKED
+            || vec->pkgs[i]->state_status == SS_HALF_INSTALLED;
         if (is_installed)
             return vec->pkgs[i];
     }
@@ -691,10 +719,33 @@ static void pkg_hash_fetch_all_installed_helper(const char *pkg_name,
     }
 }
 
-void pkg_hash_fetch_all_installed(pkg_vec_t * all)
+static void pkg_hash_fetch_all_half_or_installed_helper(const char *pkg_name,
+                                                        void *entry, void *data)
+{
+    abstract_pkg_t *ab_pkg = (abstract_pkg_t *) entry;
+    pkg_vec_t *all = (pkg_vec_t *) data;
+    pkg_vec_t *pkg_vec = ab_pkg->pkgs;
+    unsigned int j;
+
+    if (!pkg_vec)
+        return;
+
+    for (j = 0; j < pkg_vec->len; j++) {
+        pkg_t *pkg = pkg_vec->pkgs[j];
+        int is_installed = pkg->state_status == SS_INSTALLED
+                           || pkg->state_status == SS_UNPACKED
+                           || pkg->state_status == SS_HALF_INSTALLED;
+        if (is_installed)
+            pkg_vec_insert(all, pkg);
+    }
+}
+
+void pkg_hash_fetch_all_installed(pkg_vec_t * all, int include_half_installed)
 {
     hash_table_foreach(&opkg_config->pkg_hash,
-                       pkg_hash_fetch_all_installed_helper, all);
+            include_half_installed ? pkg_hash_fetch_all_half_or_installed_helper
+                                   : pkg_hash_fetch_all_installed_helper,
+            all);
 }
 
 /*
