@@ -32,6 +32,7 @@
 #include "pkg_vec.h"
 #include "pkg_hash.h"
 #include "xfuncs.h"
+#include "sprintf_alloc.h"
 
 #define INITIAL_ARCH_LIST_SIZE 4
 
@@ -72,15 +73,20 @@ typedef enum job_action job_action_t;
 static libsolv_solver_t *libsolv_solver_new(void);
 static void libsolv_solver_free(libsolv_solver_t *libsolv_solver);
 static void libsolv_solver_add_job(libsolv_solver_t *libsolv_solver,
-                                   job_action_t action, char *pkg_name);
+                                   job_action_t action, const char *pkg_name,
+                                   const char *pkg_version);
 static int libsolv_solver_solve(libsolv_solver_t *libsolv_solver);
 static int libsolv_solver_execute_transaction(libsolv_solver_t *libsolv_solver);
 
 int opkg_solver_install(int num_pkgs, char **pkg_names)
 {
     int i, err;
+    char *name, *version;
+    Dataiterator di;
 
     libsolv_solver_t *solver = libsolv_solver_new();
+    if (solver == NULL)
+        return -1;
 
     if (num_pkgs == 0) {
         opkg_msg(ERROR, "No packages specified for install!\n");
@@ -88,8 +94,20 @@ int opkg_solver_install(int num_pkgs, char **pkg_names)
         goto CLEANUP;
     }
 
-    for (i = 0; i < num_pkgs; i++)
-        libsolv_solver_add_job(solver, JOB_INSTALL, pkg_names[i]);
+    for (i = 0; i < num_pkgs; i++) {
+        strip_pkg_name_and_version(pkg_names[i], &name, &version);
+
+        dataiterator_init(&di, solver->pool, solver->repo_available, 0,
+                          SOLVABLE_NAME | SOLVABLE_PROVIDES, name, SEARCH_GLOB);
+        while (dataiterator_step(&di)) {
+            libsolv_solver_add_job(solver, JOB_INSTALL, di.kv.str, version);
+            dataiterator_skip_solvable(&di);
+        }
+
+        dataiterator_free(&di);
+        free(name);
+        free(version);
+    }
 
     err = libsolv_solver_solve(solver);
     if (err)
@@ -105,8 +123,11 @@ CLEANUP:
 int opkg_solver_remove(int num_pkgs, char **pkg_names)
 {
     int i, err;
+    Dataiterator di;
 
     libsolv_solver_t *solver = libsolv_solver_new();
+    if (solver == NULL)
+        return -1;
 
     if (num_pkgs == 0) {
         opkg_msg(ERROR, "No packages specified for removal!\n");
@@ -114,8 +135,14 @@ int opkg_solver_remove(int num_pkgs, char **pkg_names)
         goto CLEANUP;
     }
 
-    for (i = 0; i < num_pkgs; i++)
-        libsolv_solver_add_job(solver, JOB_REMOVE, pkg_names[i]);
+    for (i = 0; i < num_pkgs; i++){
+        dataiterator_init(&di, solver->pool, solver->repo_installed, 0, 0, pkg_names[i], SEARCH_GLOB);
+        while (dataiterator_step(&di)) {
+            libsolv_solver_add_job(solver, JOB_REMOVE, di.kv.str, NULL);
+            dataiterator_skip_solvable(&di);
+        }
+        dataiterator_free(&di);
+    }
 
     err = libsolv_solver_solve(solver);
     if (err)
@@ -131,14 +158,23 @@ CLEANUP:
 int opkg_solver_upgrade(int num_pkgs, char **pkg_names)
 {
     int i, err;
+    Dataiterator di;
 
     libsolv_solver_t *solver = libsolv_solver_new();
+    if (solver == NULL)
+        return -1;
 
     if (num_pkgs == 0) {
-        libsolv_solver_add_job(solver, JOB_UPGRADE, 0);
+        libsolv_solver_add_job(solver, JOB_UPGRADE, 0, NULL);
     } else {
-        for (i = 0; i < num_pkgs; i++)
-            libsolv_solver_add_job(solver, JOB_UPGRADE, pkg_names[i]);
+        for (i = 0; i < num_pkgs; i++) {
+            dataiterator_init(&di, solver->pool, solver->repo_installed, 0, 0, pkg_names[i], SEARCH_GLOB);
+            while (dataiterator_step(&di)) {
+                libsolv_solver_add_job(solver, JOB_UPGRADE, di.kv.str, NULL);
+                dataiterator_skip_solvable(&di);
+            }
+            dataiterator_free(&di);
+        }
     }
 
     err = libsolv_solver_solve(solver);
@@ -158,12 +194,14 @@ int opkg_solver_distupgrade(int num_pkgs, char **pkg_names)
     int i, err;
 
     libsolv_solver_t *solver = libsolv_solver_new();
+    if (solver == NULL)
+        return -1;
 
     if (num_pkgs == 0) {
-        libsolv_solver_add_job(solver, JOB_DISTUPGRADE, 0);
+        libsolv_solver_add_job(solver, JOB_DISTUPGRADE, 0, NULL);
     } else {
         for (i = 0; i < num_pkgs; i++)
-            libsolv_solver_add_job(solver, JOB_DISTUPGRADE, pkg_names[i]);
+            libsolv_solver_add_job(solver, JOB_DISTUPGRADE, pkg_names[i], NULL);
     }
 
     err = libsolv_solver_solve(solver);
@@ -257,11 +295,11 @@ static void libsolv_solver_set_arch_policy(libsolv_solver_t *libsolv_solver)
    into an id usable by libsolv. */
 static Id dep2id(Pool *pool, compound_depend_t *dep)
 {
-    Id nameId = NULL;
-    Id versionId = NULL;
+    Id nameId = 0;
+    Id versionId = 0;
     Id flagId = 0;
-    Id previousId = NULL;
-    Id dependId = NULL;
+    Id previousId = 0;
+    Id dependId = 0;
 
     int i;
 
@@ -426,7 +464,7 @@ static void populate_installed_repo(libsolv_solver_t *libsolv_solver)
 
     pkg_vec_t *installed_pkgs = pkg_vec_alloc();
 
-    pkg_hash_fetch_all_installed(installed_pkgs);
+    pkg_hash_fetch_all_installed(installed_pkgs, 1);
 
     for (i = 0; i < installed_pkgs->len; i++) {
         pkg_t *pkg = installed_pkgs->pkgs[i];
@@ -484,7 +522,8 @@ static void populate_available_repos(libsolv_solver_t *libsolv_solver)
 
         /* if the package is installed or unpacked, skip it */
         if (pkg->state_status == SS_INSTALLED ||
-            pkg->state_status == SS_UNPACKED)
+            pkg->state_status == SS_UNPACKED ||
+            pkg->state_status == SS_HALF_INSTALLED)
             continue;
 
         /* if the package is marked as held, skip it */
@@ -530,7 +569,7 @@ static void populate_available_repos(libsolv_solver_t *libsolv_solver)
     pkg_vec_free(available_pkgs);
 }
 
-static void libsolv_solver_init(libsolv_solver_t *libsolv_solver)
+static int libsolv_solver_init(libsolv_solver_t *libsolv_solver)
 {
     /* initialize the solver job queue */
     queue_init(&libsolv_solver->solver_jobs);
@@ -559,6 +598,12 @@ static void libsolv_solver_init(libsolv_solver_t *libsolv_solver)
     pool_set_flag(libsolv_solver->pool,
                   POOL_FLAG_IMPLICITOBSOLETEUSESPROVIDES, 1);
 
+    /* Use version matching that most closely matches debian */
+    if (pool_setdisttype(libsolv_solver->pool, DISTTYPE_DEB) == -1) {
+        opkg_message(ERROR, "libsolv not built with Debian or Multi semantics\n");
+        return -1;
+    }
+
     /* read in repo of installed packages */
     populate_installed_repo(libsolv_solver);
 
@@ -590,45 +635,53 @@ static void libsolv_solver_init(libsolv_solver_t *libsolv_solver)
         solver_set_flag(libsolv_solver->solver,
                         SOLVER_FLAG_NO_INFARCHCHECK, 1);
     }
+
+    return 0;
 }
 
 static libsolv_solver_t *libsolv_solver_new(void)
 {
     libsolv_solver_t *libsolv_solver;
+    int err;
 
     libsolv_solver = xcalloc(1, sizeof(libsolv_solver_t));
-    libsolv_solver_init(libsolv_solver);
+    err = libsolv_solver_init(libsolv_solver);
+    if (err) {
+        opkg_message(ERROR, "Could not initialize libsolv solver\n");
+        libsolv_solver_free(libsolv_solver);
+        return NULL;
+    }
 
     return libsolv_solver;
 }
 
 static void libsolv_solver_add_job(libsolv_solver_t *libsolv_solver,
-                                   job_action_t action, char *pkg_name)
+                                   job_action_t action, const char *pkg_name,
+                                   const char *pkg_version)
 {
     Id what = 0;
     Id how = 0;
-    char *name, *version;
 
-    strip_pkg_name_and_version(pkg_name, &name, &version);
-    if (version) {
+    if (pkg_version) {
         what = pool_rel2id(libsolv_solver->pool,
-                           pool_str2id(libsolv_solver->pool, name, 1),
-                           pool_str2id(libsolv_solver->pool, version, 1),
+                           pool_str2id(libsolv_solver->pool, pkg_name, 1),
+                           pool_str2id(libsolv_solver->pool, pkg_version, 1),
                            REL_EQ, 1);
     } else {
-        what = pool_str2id(libsolv_solver->pool, name, 1);
+        what = pool_str2id(libsolv_solver->pool, pkg_name, 1);
     }
 
     switch (action) {
     case JOB_INSTALL:
-        how = SOLVER_INSTALL | SOLVER_SOLVABLE_NAME;
+        how = SOLVER_INSTALL | SOLVER_SOLVABLE_PROVIDES;
         break;
     case JOB_REMOVE:
+        /* Only remove packages by real package name -- matches internalsolv */
         how = SOLVER_ERASE | SOLVER_SOLVABLE_NAME;
         break;
     case JOB_UPGRADE:
         if (pkg_name && strcmp(pkg_name, "") != 0) {
-            how = SOLVER_UPDATE | SOLVER_SOLVABLE_NAME | SOLVER_TARGETED;
+            how = SOLVER_UPDATE | SOLVER_SOLVABLE_PROVIDES | SOLVER_TARGETED;
         } else {
             how = SOLVER_UPDATE | SOLVER_SOLVABLE_REPO;
             what = libsolv_solver->pool->installed->repoid;
@@ -636,7 +689,7 @@ static void libsolv_solver_add_job(libsolv_solver_t *libsolv_solver,
         break;
     case JOB_DISTUPGRADE:
         if (pkg_name && strcmp(pkg_name, "") != 0) {
-            how = SOLVER_DISTUPGRADE | SOLVER_SOLVABLE_NAME;
+            how = SOLVER_DISTUPGRADE | SOLVER_SOLVABLE_PROVIDES;
         } else {
             how = SOLVER_DISTUPGRADE | SOLVER_SOLVABLE_ALL;
             queue_push2(&libsolv_solver->solver_jobs, SOLVER_DROP_ORPHANED | SOLVER_SOLVABLE_ALL, 0);
@@ -652,8 +705,12 @@ static void libsolv_solver_add_job(libsolv_solver_t *libsolv_solver,
 
     queue_push2(&libsolv_solver->solver_jobs, how, what);
 
-    free(name);
-    free(version);
+    /* Given two packages, one which provides 'name' and one which is
+       actually named 'name', prefer the latter. Regression for issue9533 */
+    if ((how & SOLVER_SELECTMASK) == SOLVER_SOLVABLE_PROVIDES) {
+        queue_push2(&libsolv_solver->solver_jobs,
+                    SOLVER_FAVOR | SOLVER_SOLVABLE_NAME, what);
+    }
 }
 
 static int libsolv_solver_solve(libsolv_solver_t *libsolv_solver)
@@ -691,7 +748,8 @@ static int libsolv_solver_solve(libsolv_solver_t *libsolv_solver)
 
 static void libsolv_solver_free(libsolv_solver_t *libsolv_solver)
 {
-    solver_free(libsolv_solver->solver);
+    if (libsolv_solver->solver)
+        solver_free(libsolv_solver->solver);
     queue_free(&libsolv_solver->solver_jobs);
     pool_free(libsolv_solver->pool);
     free(libsolv_solver);
@@ -760,7 +818,6 @@ static int libsolv_solver_execute_transaction(libsolv_solver_t *libsolv_solver)
 
             switch (typeId) {
             case SOLVER_TRANSACTION_ERASE:
-                opkg_message(NOTICE, "Removing pkg %s - %s\n", pkg->name, pkg->version);
                 ret = opkg_remove_pkg(pkg);
                 break;
             case SOLVER_TRANSACTION_DOWNGRADE:
@@ -776,7 +833,13 @@ static int libsolv_solver_execute_transaction(libsolv_solver_t *libsolv_solver)
                                     != SOLVER_RULE_JOB)
                     pkg->auto_installed = 1;
 
-                opkg_message(NOTICE, "Installing pkg %s - %s\n", pkg->name, pkg->version);
+                if (pkg->dest == NULL)
+                    pkg->dest = opkg_config->default_dest;
+
+                if (!opkg_config->download_only) {
+                    opkg_message(NOTICE, "Installing %s (%s) on %s\n",
+                                 pkg->name, pkg->version, pkg->dest->name);
+                }
                 ret = opkg_install_pkg(pkg, 0);
                 break;
             case SOLVER_TRANSACTION_UPGRADE:
@@ -785,16 +848,26 @@ static int libsolv_solver_execute_transaction(libsolv_solver_t *libsolv_solver)
                 /* if an old version was found set the new package's
                    autoinstalled status to that of the old package. */
                 if (old) {
-                    char *old_version = pkg_version_str_alloc(old);
-
                     pkg->auto_installed = old->auto_installed;
-                    opkg_message(NOTICE, "Upgrading pkg %s from %s to %s\n",
-                                 pkg->name, old_version, pkg->version);
+                    pkg->dest = old->dest;
+                    if (pkg->dest == NULL)
+                        pkg->dest = opkg_config->default_dest;
 
-                    free(old_version);
+                    if (!opkg_config->download_only) {
+                        char *old_version = pkg_version_str_alloc(old);
+
+                        opkg_message(NOTICE, "Upgrading %s from %s to %s on %s\n",
+                                     pkg->name, old_version, pkg->version, pkg->dest->name);
+
+                        free(old_version);
+                    }
                 } else {
-                    opkg_message(NOTICE, "Upgrading pkg %s to %s\n",
-                                 pkg->name, pkg->version);
+                    if (pkg->dest == NULL)
+                        pkg->dest = opkg_config->default_dest;
+                    if (!opkg_config->download_only) {
+                        opkg_message(NOTICE, "Upgrading %s to %s on %s\n",
+                                     pkg->name, pkg->version, pkg->dest->name);
+                    }
                 }
 
                 ret = opkg_install_pkg(pkg, 1);
@@ -814,3 +887,11 @@ CLEANUP:
     transaction_free(transaction);
     return err;
 }
+
+char *opkg_solver_version_alloc(void)
+{
+    char *version;
+
+    sprintf_alloc(&version, "libsolv %s", solv_version);
+    return version;
+}
