@@ -67,11 +67,6 @@ void str_list_append(str_list_t * list, char *data)
     void_list_append((void_list_t *) list, xstrdup(data));
 }
 
-void str_list_push(str_list_t * list, char *data)
-{
-    void_list_push((void_list_t *) list, xstrdup(data));
-}
-
 str_list_elt_t *str_list_pop(str_list_t * list)
 {
     return (str_list_elt_t *) void_list_pop((void_list_t *) list);
@@ -100,23 +95,12 @@ str_list_elt_t *str_list_first(str_list_t * list)
     return (str_list_elt_t *) void_list_first((void_list_t *) list);
 }
 
-str_list_elt_t *str_list_prev(str_list_t * list, str_list_elt_t * node)
-{
-    return (str_list_elt_t *) void_list_prev((void_list_t *) list,
-                                             (void_list_elt_t *) node);
-}
-
 str_list_elt_t *str_list_next(str_list_t * list, str_list_elt_t * node)
 {
     return (str_list_elt_t *) void_list_next((void_list_t *) list,
                                              (void_list_elt_t *) node);
 }
 
-str_list_elt_t *str_list_last(str_list_t * list)
-{
-    return (str_list_elt_t *) void_list_last((void_list_t *) list);
-}
-
 void str_list_purge(str_list_t * list)
 {
     str_list_deinit(list);
