@@ -1,3 +1,4 @@
+/* vi: set expandtab sw=4 sts=4: */
 /* xregex.c - regex functions with error messages
 
    Carl D. Worth
@@ -15,23 +16,28 @@
    GNU General Public License for more details.
 */
 
+#include "config.h"
+
+#include <malloc.h>
+
+#include "opkg_message.h"
 #include "xregex.h"
-#include "libbb/libbb.h"
+#include "xfuncs.h"
 
-static void print_regcomp_err(const regex_t *preg, int err);
+static void print_regcomp_err(const regex_t * preg, int err);
 
-int xregcomp(regex_t *preg, const char *regex, int cflags)
+int xregcomp(regex_t * preg, const char *regex, int cflags)
 {
     int err;
     err = regcomp(preg, regex, cflags);
     if (err) {
-	print_regcomp_err(preg, err);
+        print_regcomp_err(preg, err);
     }
 
     return err;
 }
 
-static void print_regcomp_err(const regex_t *preg, int err)
+static void print_regcomp_err(const regex_t * preg, int err)
 {
     unsigned int size;
     char *error;
