@@ -1,4 +1,4 @@
-#! /usr/bin/env python
+#! /usr/bin/env python3
 #
 # Install a package 'x' which PROVIDES 'v'. Then try to install 'y' which
 # PROVIDES and CONFLICTS 'v', indicating that no other provider of the virtual
@@ -25,4 +25,4 @@ if not opkgcl.is_installed("x"):
 
 opkgcl.install("y")
 if opkgcl.is_installed("y"):
-	opk.xfail("Package 'y' installed despite conflict with 'v' provided by 'x'.")
+	opk.xfail("[internalsolv] Package 'y' installed despite conflict with 'v' provided by 'x'.")
