@@ -1,4 +1,4 @@
-import tarfile, os, sys
+import tarfile, os, sys, stat
 import cfg
 import errno
 
@@ -12,6 +12,12 @@ class Opk:
 			"Description", "OE", "Homepage", "Priority",\
 			"Conffiles", "Essential"]
 
+	control = None
+	preinst = None
+	postinst = None
+	prerm = None
+	postrm = None
+
 	def __init__(self, **control):
 		for k in control.keys():
 			if k not in self.valid_control_fields:
@@ -26,46 +32,80 @@ class Opk:
 			control["Version"] = "1.0"
 		self.control = control
 
-	def write(self, tar_not_ar=False, data_files=None):
+	def write(self, tar_not_ar=False, data_files=None, compression='gz'):
+		COMPRESSORS = ['gz', 'bz2', 'xz']
+		if compression not in COMPRESSORS:
+			raise Exception("Invalid compression type: "
+				"{}. Supported compressors: {}"\
+				.format(compression, ", ".join(COMPRESSORS)))
+
+		control_file = 'control.tar.' + compression
+		data_file = 'data.tar.' + compression
+		tar_mode = "w:" + compression
+
+		TEMP_FILES = ['control', control_file, data_file,
+			'preinst', 'postinst', 'prerm', 'postrm',
+			'debian-binary']
+
 		filename = "{Package}_{Version}_{Architecture}.opk"\
 						.format(**self.control)
-		if os.path.exists(filename):
-			os.unlink(filename)
-		if os.path.exists("control"):
-			os.unlink("control")
-		if os.path.exists("control.tar.gz"):
-			os.unlink("control.tar.gz")
-		if os.path.exists("data.tar.gz"):
-			os.unlink("data.tar.gz")
 
-		f = open("control", "w")
-		for k in self.control.keys():
-			f.write("{}: {}\n".format(k, self.control[k]))
-		f.close()
+		for f in TEMP_FILES + [filename]:
+			if os.path.exists(f):
+				os.unlink(f)
 
-		tar = tarfile.open("control.tar.gz", "w:gz")
-		tar.add("control")
-		tar.close()
+		with open("debian-binary", "w") as f:
+			f.write("2.0\n")
 
-		tar = tarfile.open("data.tar.gz", "w:gz")
-		if data_files:
-			for df in data_files:
-				tar.add(df)
-		tar.close()
+		with open("control", "w") as f:
+			for k in self.control.keys():
+				f.write("{}: {}\n".format(k, self.control[k]))
 
+		if self.preinst:
+			with open("preinst", "w") as f:
+				os.fchmod(f.fileno(), 0o755)
+				f.write(self.preinst)
+
+		if self.postinst:
+			with open("postinst", "w") as f:
+				os.fchmod(f.fileno(), 0o755)
+				f.write(self.postinst)
+
+		if self.prerm:
+			with open("prerm", "w") as f:
+				os.fchmod(f.fileno(), 0o755)
+				f.write(self.prerm)
+
+		if self.postrm:
+			with open("postrm", "w") as f:
+				os.fchmod(f.fileno(), 0o755)
+				f.write(self.postrm)
+
+		with tarfile.open(control_file, tar_mode) as tar:
+			tar.add("control")
+			if self.preinst: tar.add("preinst")
+			if self.postinst: tar.add("postinst")
+			if self.prerm: tar.add("prerm")
+			if self.postrm: tar.add("postrm")
+
+		with tarfile.open(data_file, tar_mode) as tar:
+			if data_files:
+				for df in data_files:
+					tar.add(df)
 
 		if tar_not_ar:
-			tar = tarfile.open(filename, "w:gz")
-			tar.add("control.tar.gz")
-			tar.add("data.tar.gz")
-			tar.close()
+			with tarfile.open(filename, tar_mode) as tar:
+				tar.add(control_file)
+				tar.add(data_file)
 		else:
-			os.system("ar q {} control.tar.gz data.tar.gz \
-					2>/dev/null".format(filename))
+			os.system("ar q {0} {1} {2} {3}\
+					2>/dev/null".format(filename,
+						'debian-binary',
+						control_file, data_file))
 
-		os.unlink("control")
-		os.unlink("control.tar.gz")
-		os.unlink("data.tar.gz")
+		for f in TEMP_FILES:
+			if os.path.exists(f):
+				os.unlink(f)
 
 import hashlib
 def md5sum_file(fname):
@@ -86,9 +126,9 @@ class OpkGroup:
 	def addOpk(self, opk):
 		self.opk_list.append(opk)
 
-	def write_opk(self, tar_not_ar=False):
+	def write_opk(self, tar_not_ar=False, compression='gz'):
 		for o in self.opk_list:
-			o.write(tar_not_ar)
+			o.write(tar_not_ar=tar_not_ar, compression=compression)
 
 	def write_list(self, filename="Packages"):
 		f = open(filename, "w")
@@ -131,8 +171,9 @@ def regress_init():
 
 	os.system("rm -fr {}".format(cfg.offline_root))
 
-	os.makedirs("{}/etc/opkg".format(cfg.offline_root))
-	f = open("{}/etc/opkg/opkg.conf".format(cfg.offline_root), "w")
+	confdir=os.environ['SYSCONFDIR']+"/opkg";
+	os.makedirs(("{}"+confdir).format(cfg.offline_root))
+	f = open(("{}"+confdir+"/opkg.conf").format(cfg.offline_root), "w")
 	f.write("arch all 1\n")
 	f.write("src test file:{}\n".format(cfg.opkdir))
 	f.close()
