@@ -1,4 +1,4 @@
-#!/usr/bin/python3
+#! /usr/bin/env python3
 #
 # Reporter: alexeytech@gmail.com
 #
@@ -56,6 +56,6 @@ if opkgcl.is_installed("a"):
 
 x2fullpath = "{}/x2".format(cfg.offline_root)
 if not x2fullpath in opkgcl.files("b"):
-        opk.xfail("Package 'b' does not own file 'x2'.")
+        opk.xfail("[libsolv] Package 'b' does not own file 'x2'.")
 
 opkgcl.remove("b")
